#pragma once

struct C3dPoint {
	double x,y,z;

	C3dPoint() {}
	C3dPoint(double a, double b, double c): x(a), y(b), z(c) {}
	C3dPoint operator +=(const C3dPoint& p) { x+=p.x; y+=p.y; z+=p.z; return *this; }
	C3dPoint operator -=(const C3dPoint& p) { x-=p.x; y-=p.y; z-=p.z; return *this; }
	C3dPoint operator *(double d) { return C3dPoint(d*x, d*y, d*z); }
	C3dPoint operator *=(double d) { x*=d, y*=d, z*=d; return *this; }
	C3dPoint operator +(const C3dPoint& p) const { C3dPoint tp(*this); tp+=p; return tp; }
	C3dPoint operator -(const C3dPoint& p) const { C3dPoint tp(*this); tp-=p; return tp; }
	C3dPoint zero() { x=y=z=0; return *this; }
	double abs() const { return sqrt(x*x+y*y+z*z); }
	double *GetBaseAddr() { return &x; }
	bool IsZero() const { return x==0 && y==0 && z==0; }
	bool Normalize() {
		double fv = sqrt(x*x+y*y+z*z);
		if(fv != 0) { x/=fv, y/=fv, z/=fv; return true; } else return false;
	}
};

struct C3dLine {
	C3dPoint from, to;
	C3dLine() {}
	C3dLine(double a, double b, double c, double d, double e, double f):
		from(a,b,c), to(d,e,f) {}
};

struct C3dMatrix {
	enum InitType { noinit, identity, allzero };
	union {
		double m[3][3];
		struct {
			double m00, m01, m02, m10, m11, m12, m20, m21, m22;
		};
	};
	C3dMatrix(int type=noinit);
	C3dMatrix(double a, double b, double c, double d, double e, double f, double g, double h, double i):
		m00(a), m01(b), m02(c), m10(d), m11(e), m12(f), m20(g), m21(h), m22(i)
	{
	}
	C3dMatrix operator *(const C3dMatrix& p) const;
	const C3dMatrix& operator *=(const C3dMatrix& p)
	{
		C3dMatrix a(*this*p); *this=a; return *this;
	}
	C3dPoint operator *(const C3dPoint& p) const
	{
		return C3dPoint(m[0][0]*p.x+m[0][1]*p.y+m[0][2]*p.z, m[1][0]*p.x+m[1][1]*p.y+m[1][2]*p.z,
			m[2][0]*p.x+m[2][1]*p.y+m[2][2]*p.z);
	}
};

class CMy3DDoc : public CDocument
{
protected: // create from serialization only
	CMy3DDoc();
	int m_nPt; C3dLine *m_Line;
	DECLARE_DYNCREATE(CMy3DDoc)
	DECLARE_MESSAGE_MAP()

// Attributes
public:
	int GetCount() const { return m_nPt; }
	const C3dLine& LineAt(int x) const { return m_Line[x]; }

	virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual void DeleteContents();

public:
	virtual ~CMy3DDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
};
