#include "stdafx.h"
#include "3D.h"
#include "MainFrm.h"
#include "3DDoc.h"
#include ".\3dview.h"

IMPLEMENT_DYNCREATE(CMy3DView, CView)
BEGIN_MESSAGE_MAP(CMy3DView, CView)
	ON_WM_KEYDOWN()
	ON_WM_SIZE()
	ON_WM_KEYUP()
	ON_WM_KILLFOCUS()

	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_VIEW_FREEFORM, OnViewFreeform)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FREEFORM, OnUpdateViewFreeform)
	ON_COMMAND(ID_VIEW_PERSPECTIVE, OnViewPerspective)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PERSPECTIVE, OnUpdateViewPerspective)
	ON_COMMAND(ID_VIEW_SET_CENTER, OnViewSetCenter)
	ON_COMMAND(ID_VIEW_CENTER_FOCUS, OnViewCenterFocus)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CENTER_FOCUS, OnUpdateViewCenterFocus)
	ON_WM_SYSCOLORCHANGE()
	ON_WM_SETFOCUS()
END_MESSAGE_MAP()

int CMy3DView::m_nKeys; BYTE CMy3DView::m_bKey[256];

#define CorrectAbs1(x) if(x<-1) x=-1; else if(x>1) x=1
CMy3DView::CMy3DView() : m_sizeDC(0,0), m_pCurBmp(NULL), m_pOldBmp(NULL),
	m_szScr(0,0), m_dwLastTime(0),
	m_bFreeForm(false), m_bPersp(true), m_bFocused(false),
	m_dblPersf(120), m_dblScrn(200),
	m_matAngInv(C3dMatrix::identity), m_matAng(C3dMatrix::identity)
{
}

CMy3DView::~CMy3DView()
{
	m_SrcDC.SelectObject(m_pOldBmp); delete m_pCurBmp;
}

BOOL CMy3DView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/* Necessity of the manual coordinate clipping:
On Windows 9x, all coordinates are considered 16-bits.
The following code will draw a line to the right edge.
pDC->MoveTo(100,100); pDC->LineTo(30000,100);

However, this will draw to the left edge, since 32769 is considered as -1!
pDC->MoveTo(100,100); pDC->LineTo(32769,100); */
class CClippedLine {
	CDC *m_pDC; const CRect& m_Rct; double m_sx, m_sy;
	CRect m_TmpRct;
public:
	CClippedLine(CDC *pDC, const CRect& rc): m_pDC(pDC), m_Rct(rc) {}
	CClippedLine(CDC *pDC, const CSize& sz): m_pDC(pDC), m_Rct(m_TmpRct)
	{
		m_TmpRct.left=m_TmpRct.top=0; m_TmpRct.right=sz.cx, m_TmpRct.bottom=sz.cy;
	}
	void MoveTo(double x, double y) { m_sx=x; m_sy=y; }
	void LineTo(double x, double y);
};

void CClippedLine::LineTo(double x, double y)
{
	double px=x, py=y;
#if 1
	//Both side beyond the range; do not display it at all.
	if((m_sx<0 && x<0) || (m_sy<0 && y<0) || (m_sx>=m_Rct.right && x>=m_Rct.right)
		|| (m_sy>=m_Rct.bottom && y>=m_Rct.bottom)) goto failure;

	//TRACE("(%.1f %.1f)-(%.1f %.1f) ", m_sx, m_sy, x,  y);
	if(m_sx<0) { //clip m_sx at point 0
		//(-3,5)sx,sy - (0, ??) - (4,12)x,y
		m_sy=(y-m_sy)*(-m_sx)/(x-m_sx)+m_sy; m_sx=0;
	}
	if(m_sx>=m_Rct.right) { //clip m_sx at point 640
		//(30,50)x,y - (640, ??) - (930,400)sx,sy
		m_sy=(m_sy-y)*(m_Rct.right-x)/(m_sx-x)+y; m_sx=m_Rct.right;
	}
	if(m_sy<0) { //clip m_sy at point 0
		//(-3,-5)sx,sy - (??, 0) - (4,12)x,y
		m_sx=(x-m_sx)*(-m_sy)/(y-m_sy)+m_sx; m_sy=0;
	}
	if(m_sy>=m_Rct.bottom) { //clip m_sy at point 480
		//(30,50)x,y - (??, 480 - (430,600)sx,sy
		m_sx=(m_sx-x)*(m_Rct.bottom-y)/(m_sy-y)+x; m_sy=m_Rct.bottom;
	}

	if(x<0) { //clip x at point 0
		//(-3,5)x,y - (0, ??) - (4,12)sx,sy
		y=(m_sy-y)*(-x)/(m_sx-x)+y; x=0;
	}
	if(x>=m_Rct.right) {
		y=(y-m_sy)*(m_Rct.right-m_sx)/(x-m_sx)+m_sy; x=m_Rct.right;
	}
	if(y<0) {
		x=(m_sx-x)*(-y)/(m_sy-y)+x; y=0;
	}
	if(y>=m_Rct.bottom) {
		x=(x-m_sx)*(m_Rct.bottom-m_sy)/(y-m_sy)+m_sx; y=m_Rct.bottom;
	}
	//TRACE(" translated to (%.1f %.1f)-(%.1f %.1f)\n", m_sx, m_sy, x,  y);
#endif
	m_pDC->MoveTo((int)m_sx, (int)m_sy); m_pDC->LineTo((int)x, (int)y);
failure:
	m_sx=px; m_sy=py;
}

void CMy3DView::TranslatePoint(const C3dPoint& in, C3dPoint& out) const
{
	C3dPoint p(in); p-=m_dblCur; //linear transform
	if(m_bFreeForm) out=m_matAngInv*p;
	else {
		//Matrix version, result of (A*B*C)^(-1)*[[x][y][z]], where A,B,C is given below:
		double ca=cos(m_dblAngle.x), cb=cos(m_dblAngle.y), cg=cos(m_dblAngle.z),
			sa=sin(m_dblAngle.x), sb=sin(m_dblAngle.y), sg=sin(m_dblAngle.z);
		out.x=(cb*ca)*p.x          + (cb*sa)*p.y          + sb *p.z;
		out.y=(-sg*sb*ca-cg*sa)*p.x+ (-sg*sb*sa+cg*ca)*p.y+ (sg*cb)*p.z;
		out.z=(-cg*sb*ca+sg*sa)*p.x+ (-cg*sb*sa-sg*ca)*p.y+ (cg*cb)*p.z;
	}
}

//p x κ Ÿ(0  ϼ ִ)
//y  x(. 0 ߾), z  y(. 0 ߾)̴.
BOOL CMy3DView::ProjectPoint(const C3dPoint& p, double& aux, double& auy) const
{
	//The sight at 0,0 is confined to -m_dblScrn/2 ~ m_dblScrn/2, regardless
	//of the size of the screen (m_RcSize).
	if(p.x>0) { //If in front of our eyes
		if(m_bPersp) {
			double factor= m_dblPersf / p.x; //ٰ ش
			aux= m_szScrHalf.cx + (p.y*factor*m_szScr.cx/m_dblScrn);
			auy= m_szScrHalf.cy + (p.z*factor*m_szScr.cy/m_dblScrn);
		}
		else {
			aux= m_szScr.cx*(p.y/m_dblScrn + 0.5);
			auy= m_szScr.cy*(1 - p.x/m_dblScrn);
		}
		return TRUE;
	}
	return FALSE;
}

void CMy3DView::OnDraw(CDC* pDC)
{
	pDC->BitBlt(0, 0, m_szScr.cx, m_szScr.cy, &m_SrcDC, 0, 0, SRCCOPY);
}

BOOL CMy3DView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMy3DView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMy3DView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

#ifdef _DEBUG
void CMy3DView::AssertValid() const
{
	CView::AssertValid();
}

void CMy3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMy3DDoc* CMy3DView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMy3DDoc)));
	return (CMy3DDoc*)m_pDocument;
}
#endif //_DEBUG

void CMy3DView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(!m_bKey[nChar]) m_bKey[nChar]=1, m_nKeys++;
}

void CMy3DView::OnSize(UINT nType, int cx, int cy) 
{
	if(GetSafeHwnd()) {
		if(cx>m_sizeDC.cx || cy>m_sizeDC.cy) {
			CClientDC r(this);
			m_sizeDC.cx=max(m_sizeDC.cx, cx); m_sizeDC.cy=max(m_sizeDC.cy, cy);
			if(m_pCurBmp)delete m_pCurBmp;
			else {
				m_SrcDC.CreateCompatibleDC(&r);
				m_pOldBmp=m_SrcDC.GetCurrentBitmap();
			}
			m_pCurBmp=new CBitmap;
			m_pCurBmp->CreateCompatibleBitmap(&r, m_sizeDC.cx, m_sizeDC.cy);
			m_SrcDC.SelectObject(m_pCurBmp);
		}
		m_szScr=CSize(cx, cy); m_szScrHalf=CSize(cx/2, cy/2); //  
		if(cx>0 && cy>0) Redraw();
	}
	CView::OnSize(nType, cx, cy);
}

void CMy3DView::OnInitialUpdate() 
{
	//Initialize the view point
	m_dblCur=C3dPoint(-100,100,100), m_dblAngle.zero(); m_dblVeloCur.zero();
	m_dblFoc.zero();
	m_Pen.CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_WINDOWTEXT));
	CView::OnInitialUpdate(); Redraw();
}

void CMy3DView::Redraw()
{
	C3dLine l; C3dPoint t; int x; double aux, auy; CClippedLine pk(&m_SrcDC, m_szScr);
	CMy3DDoc* pDoc=GetDocument(); CPen *olp;

	olp=m_SrcDC.SelectObject(&m_Pen);
	m_SrcDC.FillSolidRect(0, 0, m_szScr.cx, m_szScr.cy, ::GetSysColor(COLOR_WINDOW));
	for(x=0;x<pDoc->GetCount();x++) {
		TranslatePoint(pDoc->LineAt(x).from, l.from);
		TranslatePoint(pDoc->LineAt(x).to, l.to);
		if(l.from.x<=0 && l.to.x<=0) continue;

		//If just one side is invisible, trim that point
		if(l.to.x<=0) { //switch it so that l.to is always x>=0
			t=l.from; l.from=l.to; l.to=t;
		}
		if(l.from.x<=0) { //conversion
			//-3,-1,-5 -> 6,7,10.. then what from would be 0:?:?... solve it!
			t.y=(l.to.y-l.from.y)*(-l.from.x)/(l.to.x-l.from.x)+l.from.y;
			t.z=(l.to.z-l.from.z)*(-l.from.x)/(l.to.x-l.from.x)+l.from.z;
			t.x=0.001; l.from=t;
		}

		ProjectPoint(l.from, aux, auy); pk.MoveTo(aux, auy);
		ProjectPoint(l.to, aux, auy); pk.LineTo(aux, auy);
	}
	m_SrcDC.SelectObject(olp);
	CClientDC dc(this);
	dc.BitBlt(0, 0, m_szScr.cx, m_szScr.cy, &m_SrcDC, 0, 0, SRCCOPY);
}

void CMy3DView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(m_bKey[nChar]) m_bKey[nChar]=0, m_nKeys--;
}

#ifndef M_PI
#define M_PI 3.141592
#endif
#define WALK 5 //ȴ ӵ. 츮 ǥ迡 1 1cm ǹѴ. (km/h)
#define TURN 840.0 //ڰ   
extern "C" DWORD WINAPI timeGetTime();
BOOL CMy3DView::CheckKey(BOOL brd)
{
	if(m_nKeys==0 && m_dblVeloCur.abs()<0.01 && !brd && m_dblPersf<=120) { //no more processing
		m_dblVeloCur.zero(); m_dwLastTime=0; return FALSE;
	}

	BOOL redraw=TRUE; DWORD it; double dblWalk, dblAngle;
	C3dPoint wlk(0,0,0), rot(0,0,0);

	it=::timeGetTime();
	if(m_dwLastTime==0) dblWalk=0, dblAngle=0;
	//36 ǹ: 1ʿ 1000 (10m), 1ð 36km
	else {
		m_dwLastTime=it-m_dwLastTime;
		dblWalk=(WALK/36.0)*m_dwLastTime, dblAngle=M_PI*m_dwLastTime/TURN;
	}

	//move command
	if(m_bKey[VK_DOWN]) wlk.x-=dblWalk; if(m_bKey[VK_UP]) wlk.x+=dblWalk; //forward/backward
	if(m_bKey['Z']) wlk.y-=dblWalk; if(m_bKey['X']) wlk.y+=dblWalk; //sidewalk
	if(m_bKey['Q']) wlk.z-=dblWalk; if(m_bKey['A']) wlk.z+=dblWalk; //up and down
	m_dblVeloCur=m_dblVeloCur*0.95+wlk*0.05;

	//Viewpoint command
	if(m_bKey[VK_LEFT]) rot.x-=dblAngle; if(m_bKey[VK_RIGHT]) rot.x+=dblAngle;
	if(m_bKey[VK_PRIOR]) rot.y-=dblAngle; if(m_bKey[VK_NEXT]) rot.y+=dblAngle;
	if(m_bKey['C']) rot.z-=dblAngle; if(m_bKey['V']) rot.z+=dblAngle;

	if(wlk.IsZero() && rot.IsZero() && m_dblVeloCur.abs()<0.01) redraw=FALSE;

	//center-the-view key
	if(m_bKey[VK_CLEAR]) {
		if(fabs(m_dblAngle.y)>0.01) m_dblAngle.y=0, redraw=TRUE;
		if(fabs(m_dblAngle.z)>0.01) m_dblAngle.z=0, redraw=TRUE;
	}
	//zoom command
	if(m_bKey[VK_INSERT]) {
		if(m_dblPersf<700) { //̹ 700    ƹ ϵ  
			redraw=TRUE, m_dblPersf+=m_dwLastTime; m_dblPersf=min(m_dblPersf, 700);
		}
	}
	else if(m_dblPersf>120) { //  ӵ  
		redraw=TRUE, m_dblPersf-=m_dwLastTime*2; m_dblPersf=max(m_dblPersf, 120);
	}

	//ӵ ִ ۼ
	if(m_bKey[VK_ESCAPE]) m_dblVeloCur.zero();

	m_dwLastTime=it;
	if(redraw || brd) {
		wlk=m_dblVeloCur;

		CString t; CMainFrame *pFrame=DYNAMIC_DOWNCAST(CMainFrame, AfxGetMainWnd());
		if(m_bFreeForm) {
			ApplyMatrix(rot); m_dblCur+=m_matAng*wlk;
			// ٽ  ٲ۴
			double c,s, cb;

			//sin(beta)   µ,   cos(beta)  acos(asin(sb))
			//ü ܿ ̰    ִ.
			s=m_matAng.m20; CorrectAbs1(s); cb=c=cos(asin(s)); //-_-
			m_dblAngle.y=CosSinToAngle(c,s);

			//cos(beta) ˸ px  cos(alpha) sin(alpha) µ, cos(beta)
			//   ο  alpha  180 ̰  ȴ.
			c=m_matAng.m00/cb; s=m_matAng.m10/cb;
			m_dblAngle.x=CosSinToAngle(c,s);

			//cos(beta) ˸ py,pz  cos(gamma), sin(gamma)  ٷ ȴ.
			//ᱹ ߿  cos(beta) ȣ̸ alpha gamma      ִ.
			//(  ϴµ alpha gamma ϰ Ǵ  ƴ϶    û)
			c=m_matAng.m22/cb; s=m_matAng.m21/cb;
			m_dblAngle.z=CosSinToAngle(c,s);
		}
		else {
			double ca=cos(m_dblAngle.x), cb=cos(m_dblAngle.y), cg=cos(m_dblAngle.z),
				sa=sin(m_dblAngle.x), sb=sin(m_dblAngle.y), sg=sin(m_dblAngle.z);
			//If shift pressed, do not consider the Y-axis.
			if(HIBYTE(::GetKeyState(VK_SHIFT))) cb=1, sb=0;
			m_dblAngle+=rot;
			double *pt; int i;
			for(i=0, pt=m_dblAngle.GetBaseAddr();i<3;i++) //have the angles always in range
				if(pt[i]>M_PI || pt[i]<-M_PI) pt[i]=CosSinToAngle(cos(pt[i]), sin(pt[i]));

			//Matrix version, result of A*B*C*[[x][y][z]], where
			//  |cos(x) -sin(x) 0|   |cos(y) 0 -sin(y)|   |1   0       0   | 
			//A=|sin(x)  cos(x) 0| B=|  0    1    0   | C=|0 cos(z) -sin(z)| 
			//  |  0       0    1|   |sin(y) 0  cos(y)|   |0 sin(z)  cos(z)| 
			m_dblCur.x+=ca*cb*wlk.x+ (-sa*cg-ca*sb*sg)*wlk.y+ (sa*sg-ca*sb*cg)*wlk.z;
			m_dblCur.y+=sa*cb*wlk.x+ (ca*cg-sa*sb*sg)*wlk.y + (-ca*sg-sa*sb*cg)*wlk.z;
			m_dblCur.z+=sb*wlk.x   + cb*sg*wlk.y            + cb*cg*wlk.z;

		}
		if(m_bFocused) { //see the original point
			//negative, and normalize
			C3dPoint dlt = m_dblFoc - m_dblCur;
			if(dlt.Normalize()) {
				double c,cb,s=dlt.z; CorrectAbs1(s); cb=c=cos(asin(s)); //-_-
				m_dblAngle.y=CosSinToAngle(c,s);

				//cos(beta) ˸ px  cos(alpha) sin(alpha) µ, cos(beta)
				//   ο  alpha  180 ̰  ȴ.
				c=dlt.x/cb; s=dlt.y/cb;
				m_dblAngle.x=CosSinToAngle(c,s);

				//m_dblAngle.z=0; // 0 ٲ ʿ 
				if(m_bFreeForm) {
					m_matAng=m_matAngInv=C3dMatrix(C3dMatrix::identity);
					ApplyMatrix(m_dblAngle); //츮  ʱ · ش
				}
			}
		}

		t.Format(_T("(%.3f, %.3f, %.3f) : (%.3f, %.3f, %.3f)%s"), m_dblCur.x, m_dblCur.y, m_dblCur.z,
			m_dblAngle.x, m_dblAngle.y, m_dblAngle.z, (m_bFreeForm ? _T("F"): _T("")));
		if(m_bFocused) {
			TCHAR fmt[100];
			_stprintf(fmt, _T(" - (%.3f, %.3f, %.3f)"), m_dblFoc.x, m_dblFoc.y, m_dblFoc.z);
			t+=fmt;
		}
		if(pFrame) pFrame->SetStatusText(t); Redraw();
	}
	return TRUE;
}

//rot:  к ()  ִ 
void CMy3DView::ApplyMatrix(const C3dPoint& rot)
{
	C3dMatrix A(C3dMatrix::identity), B(C3dMatrix::identity), C(C3dMatrix::identity);
	A.m[0][0]=A.m[1][1]=cos(rot.x); A.m[0][1]=-sin(rot.x); A.m[1][0]=-A.m[0][1];
	B.m[0][0]=B.m[2][2]=cos(rot.y); B.m[0][2]=-sin(rot.y); B.m[2][0]=-B.m[0][2];
	C.m[1][1]=C.m[2][2]=cos(rot.z); C.m[1][2]=-sin(rot.z); C.m[2][1]=-C.m[1][2];
	m_matAng*=A*B*C;
	A.m[1][0]=-A.m[1][0]; A.m[0][1]=-A.m[0][1]; //A,B,C ķ ٲ
	B.m[2][0]=-B.m[2][0]; B.m[0][2]=-B.m[0][2];
	C.m[1][2]=-C.m[1][2]; C.m[2][1]=-C.m[2][1];
	m_matAngInv=C*B*A*m_matAngInv; //  ؾ .
}

//  cosv  sinv  acos asin
//-PI    -1   0
//            Ұ     Ұ  <--   !  ʿ
//-PI/2   0  -1
//            Ұ     
//  0     1   0
//                 
// PI/2   0   1
//                 Ұ
// PI    -1   0
//̰ ؼ cos sin κ   ؿ´.
double CMy3DView::CosSinToAngle(double cosv, double sinv)
{
	//ݿø   1.000000007 ̷ ̶ 1 ˾Ƽ óϰ Ѵ.
	CorrectAbs1(cosv); CorrectAbs1(sinv);
	if(sinv>=0) return acos(cosv); else if(cosv>=0) return asin(sinv);
	else return -acos(cosv);
}

void CMy3DView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd); InitKeyInfo();
}

void CMy3DView::InitKeyInfo()
{
	memset(m_bKey, 0, sizeof(m_bKey)); m_nKeys=0;
}

#define MOUSEDELTA 120.0
void CMy3DView::OnMouseMove(UINT nFlags, CPoint point)
{
	if(GetCapture()==this) {
		CPoint delta=point-m_ptPrev; m_ptPrev=point; //ǥ ̸ ؼ
		if(m_bFreeForm) ApplyMatrix(C3dPoint(delta.x/MOUSEDELTA, delta.y/MOUSEDELTA, 0));
		else m_dblAngle.x+=delta.x/MOUSEDELTA, m_dblAngle.y+=delta.y/MOUSEDELTA;
		CheckKey(TRUE); //Ű    ȭ  ħ
	}
}

void CMy3DView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if(!m_bKey[VK_UP]) { m_nKeys++, m_bKey[VK_UP]=1; SetCapture(); m_ptPrev=point; }
}

void CMy3DView::OnLButtonUp(UINT nFlags, CPoint point)
{
	ReleaseCapture(); if(m_bKey[VK_UP]) m_nKeys--, m_bKey[VK_UP]=0;
}

void CMy3DView::OnRButtonDown(UINT nFlags, CPoint point)
{
	m_ptPrev=point; SetCapture();
}

void CMy3DView::OnRButtonUp(UINT nFlags, CPoint point)
{
	ReleaseCapture();
}

void CMy3DView::OnViewFreeform()
{
	if(m_bFreeForm) { //freeform 
	}
	else {
		m_matAng=m_matAngInv=C3dMatrix(C3dMatrix::identity);
		ApplyMatrix(m_dblAngle); //츮  ʱ · ش
	}
	m_bFreeForm^=true; CheckKey(TRUE);
}

void CMy3DView::OnUpdateViewFreeform(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_bFreeForm);
}

void CMy3DView::OnViewPerspective()
{
	m_bPersp^=true; CheckKey(TRUE);
}

void CMy3DView::OnUpdateViewPerspective(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_bPersp);
}

void CMy3DView::OnSysColorChange()
{
	CView::OnSysColorChange();
	// ٽ 
	m_Pen.DeleteObject(); m_Pen.CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_WINDOWTEXT));
	Redraw();
}

void CMy3DView::OnSetFocus(CWnd* pOldWnd)
{
	CView::OnSetFocus(pOldWnd);
	//Ŀ  ¿  帥 ð ȿȭŲ.  ʿ ۾.
	if(m_dwLastTime) m_dwLastTime=::timeGetTime();
}

void CMy3DView::OnViewSetCenter()
{
	m_dblFoc = m_dblCur; CheckKey(TRUE);
}

void CMy3DView::OnViewCenterFocus()
{
	m_bFocused^=true; CheckKey(TRUE);
}

void CMy3DView::OnUpdateViewCenterFocus(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_bFocused);
}
