#pragma once
#include "3dDoc.h"

class CMy3DView : public CView
{
protected: // create from serialization only
	CMy3DView();
	DECLARE_DYNCREATE(CMy3DView)

	//Viewer's position and speed
	C3dPoint m_dblCur, m_dblVeloCur;
	C3dPoint m_dblAngle; //normal view
	C3dPoint m_dblFoc; //focus
	C3dMatrix m_matAng, m_matAngInv; //freeform view

	//rendering option
	bool m_bFreeForm, m_bPersp, m_bFocused;
	double m_dblPersf, m_dblScrn;

	//Calculation
	void TranslatePoint(const C3dPoint& in, C3dPoint& out) const;
	BOOL ProjectPoint(const C3dPoint& pt, double& x, double& y) const;

	//virtual bitmap & GDI object
	CSize m_szScr, m_szScrHalf, m_sizeDC;
	CDC m_SrcDC; CBitmap *m_pOldBmp, *m_pCurBmp; CPen m_Pen;

	//Keyboard stuff--only one keyboard assumed (static objects)
	DWORD m_dwLastTime; //Timer
	static int m_nKeys; static BYTE m_bKey[];
	static void InitKeyInfo();
	static double CosSinToAngle(double cosv, double sinv);
	void ApplyMatrix(const C3dPoint& rot);

	//Mouse stuff
	CPoint m_ptPrev;

// Attributes
public:
	CMy3DDoc* GetDocument();

	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnInitialUpdate();

protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);

public:
	void Redraw();
	BOOL CheckKey(BOOL brd=FALSE);
	virtual ~CMy3DView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnViewFreeform();
	afx_msg void OnViewSetCenter();
	afx_msg void OnViewCenterFocus();
	afx_msg void OnUpdateViewCenterFocus(CCmdUI *pCmdUI);
	afx_msg void OnUpdateViewFreeform(CCmdUI *pCmdUI);
	afx_msg void OnViewPerspective();
	afx_msg void OnUpdateViewPerspective(CCmdUI *pCmdUI);
	afx_msg void OnSysColorChange();
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in 3DView.cpp
inline CMy3DDoc* CMy3DView::GetDocument()
   { return (CMy3DDoc*)m_pDocument; }
#endif
